import re
import logging
from time import ctime
from datetime import datetime

from django.core.management.base import BaseCommand

from common import management
from common.models import JenkinsJob, JenkinsBuild

from graphics.models import GraphicsImage, GraphicsMachine, GraphicsMetric

from graphics.util import jenkins_pull

logger = logging.getLogger('qa_dashboard')

job_names = ['openarena-benchmark',
             'gui-toolkits-benchmark',
             'nexuiz-benchmark'
             ]
machine_list = ['ps-intel-3000',
                'ps-intel-2500-le',
                'ps-intel-4000-he',
                'ps-nvidia-gt640-he',
                'ps-nvidia-gt440-le',
                'ps-nvidia-310',
                'ps-radeon-hd5750-le',
                'ps-radeon-hd7850-he',
                'ps-radeon-hd7450-le',
                ]


def add_records(jenkins_job_name):
    """
    adds openarena record to data base

    :param string jenkins job name: to grab artifact from

    example:
    >>>> add_records('openarena-benchmark-ps-intel-2500-le')
    """

    artifact_name = 'benchmark_data.json'
    last_build = [
        jenk_build.jenkins_build.build_number for jenk_build in
        GraphicsMetric.objects.filter(
            jenkins_build__job__name__exact=jenkins_job_name)]
    sorted(last_build)

    # If there is no last build give it a value of 0
    if last_build == []:
        last_build.append(0)

    logger.debug('last build: {}'.format(last_build))
    url_list = jenkins_pull.url_artifact_list(jenkins_job_name,
                                              artifact_name,
                                              last_build[-1])
    logger.debug('url_list: {}'.format(url_list))
    if url_list is not False:
        for url_file in url_list:
            jdata = management.jenkins_get(url_file, as_json=True)
            if jdata != []:
                for dic in jdata:
                    logger.debug('dic: {}'.format(dic))
                    if 'iso' in dic:
                        dt = datetime.strptime(
                            ctime(dic['time']), "%a %b %d %H:%M:%S %Y")

                        if jenkins_job_name.endswith('xmir'):
                            bench_name = "{}-{}".format(
                                'xmir', dic['name'])
                        else:
                            bench_name = "{}-{}".format(
                                'x', dic['name'])

                        if 'release' in dic:
                            release = dic['release']
                        else:
                            release = 'saucy'

                        #get_image, created_image
                        gi, ci = GraphicsImage.objects.get_or_create(
                            release=release,
                            arch='amd64',
                            variant='desktop',
                            md5="{}{}{}{}".format(
                                'saucy',
                                'amd64',
                                'desktop',
                                dic['iso'],
                            ),
                            build_number=dic['iso'],
                            image_type='daily',
                        )
                        machine = dic['machine']
                        machine_regex = re.compile(
                            ur'(ps)-([^-]*)-([^-]*)((-[^-]*)?)$'
                        )
                        m = machine_regex.match(machine)
                        if m:
                            gfx_manufacturer = m.group(2)
                            gfx_model = m.group(3)

                        #get_machine, created_machine
                        gm, cm = GraphicsMachine.objects.get_or_create(
                            name=dic['machine'],
                            gfx_manufacturer=gfx_manufacturer,
                            gfx_model=gfx_model,
                            mac_address='None',
                        )

                        #get_jenkins_job, created_jenkins_job
                        gjj, cjj = JenkinsJob.objects.get_or_create(
                            name=jenkins_job_name,
                            url="/".join(url_file.split('/')[:5])
                        )

                        #get_jenkins_build, created_jenkins_build
                        gjb, cjb = JenkinsBuild.objects.get_or_create(
                            job=gjj,
                            failed=False,
                            build_number=dic['build'],
                            ran_at=dt,
                        )

                        #get_graphics, created_graphics
                        gg, cg = GraphicsMetric.objects.get_or_create(
                            image=gi,
                            machine=gm,
                            jenkins_build=gjb,
                            name="{}-{}".format(
                                dic['machine'],
                                bench_name,
                            ),
                            minimum=dic['min'],
                            maximum=dic['max'],
                            average=dic['average'],
                            stddev=dic['stddev'],
                            ran_at=dt,
                            jenkins_url=url_file,
                            test_resolution=dic['options'],
                            framework='utah',
                        )


class Command(BaseCommand):
    help = 'add openarena artifacts to database'

    def handle(self, *args, **options):

        job_list = []
        for machine in machine_list:
            for job_name in job_names:
                logger.debug("appending {}-{} to job_list".format(
                    job_name, machine))
                job_list.append("{}-{}".format(job_name, machine))
                job_list.append("{}-{}-xmir".format(job_name, machine))
        for job in job_list:
            add_records(job)
